@@ -1,4 +1,6 @@
 ;(function($) {
+  window.link_embedder_text_gist_github_styled = 1; // custom gist styling
+
   var $goto_bottom, $input, $win;
   var $messages = $('<div/>'); // need to be defined
   var nicks = new sortedSet();
@@ -425,6 +427,7 @@
       if(e.reconnected) getNewMessages.call(document, { goto_bottom: true, silent: true });
     };
     socket.onclose = function() { $input.addClass('disabled'); };
+    socket.send('PING'); // open socket
     $input.send = function(message, attr) {
       if(message.length == 0) return $input;
       var uuid = window.guid();
